@@ -29,6 +29,11 @@ from oslo_config import types
 load_tests = testscenarios.load_tests_apply_scenarios
 
 
+def custom_type(a):
+    """Something that acts like a type, but isn't known"""
+    return a
+
+
 class GeneratorTestCase(base.BaseTestCase):
 
     groups = {
@@ -91,6 +96,10 @@ class GeneratorTestCase(base.BaseTestCase):
             deprecated_reason='This was supposed to work but it really, '
                               'really did not. Always buy house insurance.',
             help='DEPRECATED: Turn off stove'),
+        'deprecated_opt_with_deprecated_since': cfg.BoolOpt(
+            'tune_in',
+            deprecated_for_removal=True,
+            deprecated_since='13.0'),
         'deprecated_opt_with_deprecated_group': cfg.StrOpt(
             'bar', deprecated_name='foobar', deprecated_group='group1',
             help='deprecated'),
@@ -143,6 +152,9 @@ class GeneratorTestCase(base.BaseTestCase):
         'hostname_opt': cfg.HostnameOpt('hostname_opt',
                                         default='compute01.nova.site1',
                                         help='a hostname'),
+        'uri_opt': cfg.URIOpt('uri_opt',
+                              default='http://example.com',
+                              help='a URI'),
         'multi_opt': cfg.MultiStrOpt('multi_opt',
                                      default=['1', '2', '3'],
                                      help='multiple strings'),
@@ -158,9 +170,18 @@ class GeneratorTestCase(base.BaseTestCase):
         'string_type_with_bad_default': cfg.Opt('string_type_with_bad_default',
                                                 help='string with bad default',
                                                 default=4096),
+        'native_str_type': cfg.Opt('native_str_type',
+                                   help='native help',
+                                   type=str),
+        'native_int_type': cfg.Opt('native_int_type',
+                                   help='native help',
+                                   type=int),
+        'native_float_type': cfg.Opt('native_float_type',
+                                     help='native help',
+                                     type=float),
         'custom_type': cfg.Opt('custom_type',
                                help='custom help',
-                               type=type('string')),
+                               type=custom_type),
         'custom_type_name': cfg.Opt('custom_opt_type',
                                     type=types.Integer(type_name='port'
                                                        ' number'),
@@ -666,6 +687,39 @@ class GeneratorTestCase(base.BaseTestCase):
 
 # a string (string value)
 #str_opt = fooishbar
+''')),
+        ('native_str_type',
+         dict(opts=[('test', [(None, [opts['native_str_type']])])],
+              expected='''[DEFAULT]
+
+#
+# From test
+#
+
+# native help (string value)
+#native_str_type = <None>
+''')),
+        ('native_int_type',
+         dict(opts=[('test', [(None, [opts['native_int_type']])])],
+              expected='''[DEFAULT]
+
+#
+# From test
+#
+
+# native help (integer value)
+#native_int_type = <None>
+''')),
+        ('native_float_type',
+         dict(opts=[('test', [(None, [opts['native_float_type']])])],
+              expected='''[DEFAULT]
+
+#
+# From test
+#
+
+# native help (floating point value)
+#native_float_type = <None>
 ''')),
         ('multi_opt_sample_default',
          dict(opts=[('test', [(None, [opts['multi_opt_sample_default']])])],
@@ -929,7 +983,7 @@ class IgnoreDoublesTestCase(base.BaseTestCase):
         ]
 
         slurped_opts = 0
-        for _, listing in generator._list_opts(None):
+        for _, listing in generator._list_opts(['namespace']):
             for _, opts in listing:
                 slurped_opts += len(opts)
         self.assertEqual(2, slurped_opts)
@@ -1130,4 +1184,39 @@ class ChangeDefaultsTestCase(base.BaseTestCase):
         self.assertIs(orig_opt, the_opt)
 
 
+class RequiredOptionTestCase(base.BaseTestCase):
+
+    opts = [cfg.StrOpt('foo', help='foo option', default='fred'),
+            cfg.StrOpt('bar', help='bar option', required=True),
+            cfg.StrOpt('foo_bar', help='foobar'),
+            cfg.StrOpt('bars', help='bars foo', required=True)]
+
+    def test_required_option_order_single_ns(self):
+
+        config = [("namespace1", [
+                   ("alpha", self.opts)])]
+        groups = generator._get_groups(config)
+
+        fd, tmp_file = tempfile.mkstemp()
+        with open(tmp_file, 'w+') as f:
+            formatter = generator._OptFormatter(output_file=f)
+            generator._output_opts(formatter, 'alpha',
+                                   groups.pop('alpha'), True)
+        expected = '''[alpha]
+
+#
+# From namespace1
+#
+
+# bar option (string value)
+bar = <None>
+
+# bars foo (string value)
+bars = <None>
+'''
+        with open(tmp_file, 'r') as f:
+            actual = f.read()
+        self.assertEqual(expected, actual)
+
+
 GeneratorTestCase.generate_scenarios()
