@@ -18,6 +18,25 @@ import unittest
 from oslo_config import types
 
 
+class ConfigTypeTests(unittest.TestCase):
+    def test_none_concrete_class(self):
+        class MyString(types.ConfigType):
+            def __init__(self, type_name='mystring value'):
+                super(MyString, self).__init__(type_name=type_name)
+
+        self.assertRaises(TypeError, MyString)
+
+    def test_concrete_class(self):
+        class MyString(types.ConfigType):
+            def __init__(self, type_name='mystring value'):
+                super(MyString, self).__init__(type_name=type_name)
+
+            def _formatter(self, value):
+                return value
+
+        MyString()
+
+
 class TypeTestHelper(object):
     def setUp(self):
         super(TypeTestHelper, self).setUp()
@@ -551,3 +570,82 @@ class IPv6AddressTypeTests(IPAddressTypeTests):
 
     def test_ipv4_address(self):
         self.assertInvalid('192.168.0.1')
+
+
+class HostnameTypeTests(TypeTestHelper, unittest.TestCase):
+    type = types.Hostname()
+
+    def assertConvertedEqual(self, value):
+        self.assertConvertedValue(value, value)
+
+    def test_empty_hostname_fails(self):
+        self.assertInvalid('')
+
+    def test_should_return_same_hostname_if_valid(self):
+        self.assertConvertedEqual('foo.bar')
+
+    def test_trailing_quote_is_invalid(self):
+        self.assertInvalid('foo.bar"')
+
+    def test_repr(self):
+        self.assertEqual('Hostname', repr(types.Hostname()))
+
+    def test_equal(self):
+        self.assertEqual(types.Hostname(), types.Hostname())
+
+    def test_not_equal_to_other_class(self):
+        self.assertNotEqual(types.Hostname(), types.Integer())
+        self.assertNotEqual(types.Hostname(), types.String())
+
+    def test_invalid_characters(self):
+        self.assertInvalid('"host"')
+        self.assertInvalid("h'ost'")
+        self.assertInvalid("h'ost")
+        self.assertInvalid("h$ost")
+        self.assertInvalid("h%ost")
+        self.assertInvalid("host_01.co.uk")
+        self.assertInvalid("host;name=99")
+        self.assertInvalid('___site0.1001')
+        self.assertInvalid('_site01001')
+        self.assertInvalid("host..name")
+        self.assertInvalid(".host.name.com")
+        self.assertInvalid("no spaces")
+
+    def test_no_start_end_hyphens(self):
+        self.assertInvalid("-host.com")
+        self.assertInvalid("-hostname.com-")
+        self.assertInvalid("hostname.co.uk-")
+
+    def test_strip_trailing_dot(self):
+        self.assertConvertedValue('cell1.nova.site1.', 'cell1.nova.site1')
+        self.assertConvertedValue('cell1.', 'cell1')
+
+    def test_valid_hostname(self):
+        self.assertConvertedEqual('cell1.nova.site1')
+        self.assertConvertedEqual('site01001')
+        self.assertConvertedEqual('home-site-here.org.com')
+        self.assertConvertedEqual('192.168.0.1')
+        self.assertConvertedEqual('1.1.1')
+        self.assertConvertedEqual('localhost')
+
+    def test_max_segment_size(self):
+        self.assertConvertedEqual('host.%s.com' % ('x' * 63))
+        self.assertInvalid('host.%s.com' % ('x' * 64))
+
+    def test_max_hostname_size(self):
+        test_str = '.'.join('x'*31 for x in range(8))
+        self.assertEqual(255, len(test_str))
+        self.assertInvalid(test_str)
+        self.assertConvertedEqual(test_str[:-2])
+
+
+class URITypeTests(TypeTestHelper, unittest.TestCase):
+    type = types.URI()
+
+    def test_uri(self):
+        self.assertConvertedValue('http://example.com', 'http://example.com')
+        self.assertInvalid('invalid')  # it doesn't include a scheme
+        self.assertInvalid('http://')  # it doesn't include an authority
+
+    def test_repr(self):
+        self.assertEqual('URI', repr(types.URI()))
